/*
    This program finds out where the various kinds of data that a
    program can use are stored in virtual memory. This program uses
    the following kinds of storage.
      1) global uninitialized variable
      2) global initialized variable
      3) global constant data
      4) static local variable
      5) automatic (stack) local variable
      6) dynamic (heap) variable
    After this program has printed out its information, it goes into
    an infinite loop so that there is time to run the virtual memory
    mapper program to look at how the various addresses are situated
    in this program's memory map.
*/
#include <windows.h>  // Sleep()
#include <stdio.h>
#include <stdlib.h>

      int globalUnInitialized;
      int globalInitialized = 14;
const int globalConstant    = 30;

int main()
{
   // the following local variable is static (not on the stack!)
   static int staticVariable = 12;

   // the following local variable is automatically created on the stack
   char* stackVariable;

   printf("The address of the global uninitialized variable is %p\n",
            &globalUnInitialized );

   printf("The address of the global initialized variable is %p\n",
            &globalInitialized );

   printf("The address of the global constant data is %p\n",
            &globalConstant );

   printf("The address of the static local variable is %p\n",
            &staticVariable );

   printf("The address of the automatic local variable is %p\n",
            &stackVariable );

   // now make the stack variable point to something in the heap
   stackVariable = (char*)malloc(sizeof(char));

   printf("The address of the dynamic (heap) variable is %p\n",
            stackVariable);

   printf("The address of my code is %p\n",
            main );

   printf("The address of printf() seems to be %p\n",
            printf );

   printf("The address of Sleep() seems to be %p\n",
            Sleep );

   // go into an infinite loop that does not use any processor resources
   while(1){
      Sleep(1000);
   }
   return 0;
}
