/*
    This program repeatedly loops through a fairly large array in memory.

    This program uses VirtualAlloc to allocate the memory for the array.

    Use the trimPages.exe program to force this program's working set to
    zero and then watch it page the array back into physical memory.

    You can also lock all (or just some) of the pages and see what
    happens when you run trimPages.
*/
#include <windows.h>
#include <stdio.h>

// prototype for function that prints meaningful error messages
void printError(char* functionName);

#define NUMBER_OF_PAGES 10

int main()
{
   // Each page is 4096 bytes, so each page holds 1024 integers.
   int size = NUMBER_OF_PAGES*1024;
   int i;
   LPVOID saveAdd;

   // Create a name for the array.
   int* theArray;

   // Allocate NUMBER_OF_PAGES pages for the array.
   //saveAdd = VirtualAlloc( (LPVOID)0x500000,  // we decide where to put the pages
   saveAdd = VirtualAlloc( NULL,    // let the system decide where to put the pages
                           NUMBER_OF_PAGES*4096,
                           MEM_RESERVE,
                           PAGE_READWRITE );
   if (saveAdd == 0)
      printError("VirtualAlloc");

   if (!VirtualAlloc(saveAdd, NUMBER_OF_PAGES*4096, MEM_COMMIT, PAGE_READWRITE))
      printError("VirtualAlloc");

//   if (!VirtualLock(saveAdd, NUMBER_OF_PAGES*4096))
//      printError("VirtualLock");

   // Point the name at the actual array.
   theArray = (int*)saveAdd;

   while(1)
   {
      register int x = 0;
      printf(".");            // send a signal to the console
      Sleep(5000);            // give user time to trim this process's pages
      for ( ; x < 10; )       // access the code pages
         x++;
      Sleep(2000);            // access the stack pages
      for (i = 0; i < size; i += 1024)
      {
         theArray[i] = i;     // access the array pages
         Sleep(1000);
      }
   }

   return 0;
}


/*************************************************************************************
    This method is used to print out useful error messages
    for Win32 (but not C Library) function calls.
*/
void printError(char* functionName)
{
   LPVOID lpMsgBuf;
   int error_no;
   error_no = GetLastError();
   FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
   );
   // Display the string.
   fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
   fprintf(stderr, (LPTSTR)lpMsgBuf);
   //MessageBox(NULL, (LPTSTR)lpMsgBuf, "Error", MB_OK);
   // Free the buffer.
   LocalFree( lpMsgBuf );
   ExitProcess(1);  // terminate the program
}//printError
